<?php
namespace Teknisa\GOP\Services;

 use Zeedhi\Framework\DataSource\DataSet;
 use Zeedhi\Framework\DTO;
 use Zeedhi\Framework\DB\StoredProcedure\Param;
 use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
 
 class Gop10001SimulacaoValoresService {
    protected $connection;
    
    public function __construct($connection, $environment){
        $this->connection = $connection;
        $this->environment = $environment;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
    public function verifyParamVrFaturamento($row) {
        $select = "SELECT F.*, (SELECT
                        CASE NVL(MAX(IDCALCPERCENTFAT), 'FATURAMENTOLIQUIDO')
                            WHEN 'FATURAMENTO_BRUTO' THEN
                                'FATURAMENTO'
                            ELSE
                                'FATURAMENTOLIQUIDO'
                        END IDCALCPERCENTFAT
                    FROM
                        PARFILIALROP P
                    WHERE
                        P.CDFILIAL = TIPOPARAM_DRO(CDFILIAL)) VRFATURAMENTO FROM FILIAL F";
        
        $params = array (
            "P_CDFILIAL" => $row["CDFILIAL"]
        );
        
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]['IDCALCPERCENTFAT'];
        }catch (Exception $e){
            throw $e;
        }  
    } 
  
    public function updateSimulation($nrLista, $percent, $codigo, $row, $rowMonth) {
        $sqlUpdate = "DECLARE
                            V_NRSIMUORCAGOP NUMBER;
                        BEGIN
                            UPDATE SIMUORCAGOP
                            SET
                                QTORCAMENTO = :P_QTORCAMENTO,
                                VRORCAMENTO = :P_VRORCAMENTO,
                                VRPRECOBASE = :P_VRPRECOBASE,
                                VRAJUSTPERCENTUAL = :P_VRAJUSTPERCENTUAL
                            WHERE
                                CDFILIAL = :P_CDFILIAL
                                AND IDTIPOORCAOP = :P_IDTIPOORCAOP
                                AND ( CODIGO = :P_CODIGO
                                      OR CDINVEST = :P_CDINVEST
                                      OR CDIMPOSTO = :P_CDIMPOSTO
                                      OR CDSERVFILI = :P_CDSERVFILI
                                      OR CDDESPPREV = :P_CDDESPPREV
                                      OR ( CDORCAMENTOMO = :P_CDORCAMENTOMO
                                           AND CDDESPADMFAT = :P_CDDESPADMFAT )
                                      OR CDPRODUTO = :P_CDPRODUTO
                                      OR (:P_IDTIPOORCAOP IN (
                                    'DEPRECIACAO',
                                    'TAXAADM'
                                )AND :P_CODIGO IS NULL ))
                                AND DTORCAMENTO = :P_DTORCAMENTO
                                AND NRORG = :P_NRORG;
                        
                            IF ( SQL%ROWCOUNT = 0 ) THEN
                                SELECT
                                    NVL(MAX(NRSIMUORCAGOP), 0) + 1
                                INTO V_NRSIMUORCAGOP
                                FROM
                                    SIMUORCAGOP;
                        
                                INSERT INTO SIMUORCAGOP (
                                    NRSIMUORCAGOP,
                                    CDFILIAL,
                                    IDTIPOORCAOP,
                                    CODIGO,
                                    CDINVEST,
                                    CDIMPOSTO,
                                    CDSERVFILI,
                                    CDDESPPREV,
                                    CDORCAMENTOMO,
                                    CDDESPADMFAT,
                                    CDPRODUTO,
                                    QTORCAMENTO,
                                    NRDIASUTEIS,
                                    VRORCAMENTO,
                                    DTORCAMENTO,
                                    VRPRECOBASE,
                                    VRAJUSTPERCENTUAL,
                                    NRORG,
                                    CDOPERINCLUSAO,
                                    NRLISTA
                                ) VALUES (
                                    V_NRSIMUORCAGOP,
                                    :P_CDFILIAL,
                                    :P_IDTIPOORCAOP,
                                    :P_CODIGO,
                                    :P_CDINVEST,
                                    :P_CDIMPOSTO,
                                    :P_CDSERVFILI,
                                    :P_CDDESPPREV,
                                    :P_CDORCAMENTOMO,
                                    :P_CDDESPADMFAT,
                                    :P_CDPRODUTO,
                                    :P_QTORCAMENTO,
                                    :P_NRDIASUTEIS,
                                    :P_VRORCAMENTO,
                                    :P_DTORCAMENTO,
                                    :P_VRPRECOBASE,
                                    :P_VRAJUSTPERCENTUAL,
                                    :P_NRORG,
                                    :P_CDOPERINCLUSAO,
                                    :P_NRLISTA
                                );
                        
                            END IF;
                        
                        END;";
        $saveMonth = $rowMonth == null ? false : true;
        
        $params = array(
            ":P_QTORCAMENTO" => $saveMonth ? (isset($rowMonth['QTORCAMENTO']) ? str_replace(',','.',str_replace('.','',$rowMonth['QTORCAMENTO'])) : null) : null,
            ":P_VRORCAMENTO" => $saveMonth ? (isset($rowMonth['VRORCAMENTO']) ? str_replace(',','.',str_replace('.','',$rowMonth['VRORCAMENTO'])) : null) : str_replace(',','.',str_replace('.','',$row['NEW_VALUE'])),
            ":P_VRPRECOBASE" => $saveMonth ? (isset($rowMonth['VRPRECOBASE']) ? str_replace(',','.',str_replace('.','',$rowMonth['VRPRECOBASE'])) : null) : null,
            ":P_VRAJUSTPERCENTUAL" =>$saveMonth ? null : $percent,
            ":P_CDFILIAL" => $saveMonth ? $rowMonth['CDFILIAL'] : null,
            ":P_IDTIPOORCAOP" => $saveMonth ? $rowMonth['IDTIPOORCAOP'] : $row['IDTIPOORCAOP'],
            ":P_CODIGO" => $codigo ? $codigo : null,
            ":P_CDINVEST" => $saveMonth ? (isset($rowMonth['CDINVEST']) ? $rowMonth['CDINVEST'] : null) : null,
            ":P_CDIMPOSTO" => $saveMonth ? (isset($rowMonth['CDIMPOSTO']) ? $rowMonth['CDIMPOSTO'] : null) : null,
            ":P_CDSERVFILI" => $saveMonth ? (isset($rowMonth["CDSERVFILI"]) ? $rowMonth['CDSERVFILI'] : null) : null,
            ":P_CDDESPPREV" => $saveMonth ? (isset($rowMonth['CDDESPPREV']) ? $rowMonth['CDDESPPREV'] : null) : null,
            ":P_CDORCAMENTOMO" => $saveMonth ? (isset($rowMonth["CDORCAMENTOMO"]) ? $rowMonth['CDORCAMENTOMO'] : null) : null,
            ":P_CDDESPADMFAT" => $saveMonth ? (isset($rowMonth["CDDESPADMFAT"]) ? $rowMonth['CDDESPADMFAT'] : null) : null,
            ":P_CDPRODUTO" => $saveMonth ? (isset($rowMonth['CDPRODUTO']) ? $rowMonth['CDPRODUTO'] : null) : null,
            ":P_DTORCAMENTO" => $saveMonth ? $rowMonth['NRMES'] : null,
            ":P_NRDIASUTEIS" => $saveMonth ? (isset($rowMonth['NRDIASUTEIS']) ? $rowMonth['NRDIASUTEIS'] : null) : null,
            ":P_CDOPERINCLUSAO" => $this->environment->getCdOperador(),
            ":P_NRORG" => $this->environment->getNrOrg(),
            ":P_NRLISTA"=> $nrLista ? $nrLista : null
        );
         
        try{ 
            $std = $this->connection->prepare($sqlUpdate);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e;
        }
    }
    
    public function saveSimulation() {
        $sqlScript="
        
DECLARE
  V_NRANOORCAGOP  VARCHAR2(04);
  V_IDPERIODOORCA VARCHAR2(30);
  V_NRPERIODOORCA NUMBER;
BEGIN
   FOR C_BUDGET IN (SELECT X.NRSIMUORCAGOP, X.CODIGO, X.IDTIPOORCAOP, X.NRLISTA,
                           NVL(LAG(X.NRSIMUORCAGOP, 1) OVER (ORDER BY X.NRSIMUORCAGOP),0) ANTERIOR,
                           X.VRAJUSTPERCENTUAL, X.VRORCAMENTO, X.DTORCAMENTO, X.QTORCAMENTO
                      FROM (SELECT S.NRSIMUORCAGOP, S.CODIGO, S.IDTIPOORCAOP, S.NRLISTA,
                                   S.VRAJUSTPERCENTUAL, S.VRORCAMENTO, MIN(S.DTORCAMENTO) DTORCAMENTO,
                                   S.QTORCAMENTO
                              FROM SIMUORCAGOP S
                             WHERE S.CODIGO = 'BUDGETCHANGED'
                             GROUP BY S.NRSIMUORCAGOP, S.CODIGO, S.IDTIPOORCAOP, S.NRLISTA,
                                      S.VRAJUSTPERCENTUAL, S.VRORCAMENTO, S.QTORCAMENTO
                             UNION
                            SELECT MAX(S.NRSIMUORCAGOP)+1 NRSIMUORCAGOP, NULL CODIGO, NULL IDTIPOORCAOP, NULL NRLISTA,
                                   NULL VRAJUSTPERCENTUAL, NULL VRORCAMENTO, MIN(S.DTORCAMENTO) DTORCAMENTO,
                                   NULL QTORCAMENTO
                              FROM SIMUORCAGOP S) X
                     WHERE X.NRSIMUORCAGOP IS NOT NULL
                     ORDER BY X.NRSIMUORCAGOP) LOOP

      FOR C_UNIDADE IN (SELECT S.DTORCAMENTO, S.CDFILIAL
                          FROM SIMUORCAGOP S
                         WHERE S.NRSIMUORCAGOP BETWEEN C_BUDGET.ANTERIOR AND C_BUDGET.NRSIMUORCAGOP-1
                         GROUP BY S.DTORCAMENTO, S.CDFILIAL
                         ORDER BY S.CDFILIAL, S.DTORCAMENTO) LOOP
    
         BEGIN
            SELECT TO_CHAR(P.DTINIPERIORCA,'YYYY'), P.IDPERIODOORCA, P.NRPERIODO
              INTO V_NRANOORCAGOP, V_IDPERIODOORCA, V_NRPERIODOORCA
              FROM PERIODOORCAGOP P
             WHERE P.CDFILIAL = C_UNIDADE.CDFILIAL
               AND C_UNIDADE.DTORCAMENTO BETWEEN P.DTINIPERIORCA AND P.DTFIMPERIORCA;
         EXCEPTION
           WHEN NO_DATA_FOUND THEN
             V_NRANOORCAGOP := NULL; V_IDPERIODOORCA := NULL; V_NRPERIODOORCA := NULL;
         END;
                                     
         UPDATE (SELECT D.CDSERVFILI, D.VRPRECOBASE, D.NRDIASUTEIS, D.VRORCAMENTO, D.QTORCAMENTO, D.VRPERCENTUAL,
                        O.CDFILIAL, I.IDTIPOORCAOP, M.DTORCAMENTO, D.CDPRODUTO, I.CDDESPPREV, D.CDDESPADMFAT, D.CDORCAMENTOMO, 
                        CASE WHEN I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'MAOOBRA') 
                             THEN 1
                             ELSE (SELECT CASE WHEN COUNT(1) = 0 THEN 1 ELSE COUNT(1) END
                                     FROM ITDETALORCAMESGOP T
                                    WHERE T.NRITORCAMESGOP = M.NRITORCAMESGOP)
                        END FATOR
                   FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                               AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                      'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                       INNER JOIN ITORCAMESGOP M ON I.NRITEMORCAOP = M.NRITEMORCAOP
                                       INNER JOIN ITDETALORCAMESGOP D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                  WHERE O.CDFILIAL = C_UNIDADE.CDFILIAL
                    AND O.NRANOORCAGOP = V_NRANOORCAGOP
                    AND O.IDPERIODOORCA = V_IDPERIODOORCA
                    AND O.NRPERIODOORCA = V_NRPERIODOORCA) X
            SET X.VRORCAMENTO = (SELECT VRORCAMENTO
                                   FROM SIMUORCAGOP S
                                  WHERE S.CDFILIAL = X.CDFILIAL
                                    AND S.IDTIPOORCAOP = X.IDTIPOORCAOP
                                    AND ((S.CDSERVFILI = X.CDSERVFILI) OR (S.CDPRODUTO = X.CDPRODUTO) OR (S.CDDESPPREV = X.CDDESPPREV) 
                                       OR (X.CDDESPADMFAT = S.CDDESPADMFAT AND X.CDORCAMENTOMO = S.CDORCAMENTOMO))
                                    AND S.DTORCAMENTO = X.DTORCAMENTO
                                    AND S.NRSIMUORCAGOP BETWEEN C_BUDGET.ANTERIOR AND C_BUDGET.NRSIMUORCAGOP-1)/X.FATOR,
                X.VRPRECOBASE = (SELECT VRPRECOBASE
                                   FROM SIMUORCAGOP S
                                  WHERE S.CDFILIAL = X.CDFILIAL
                                    AND S.IDTIPOORCAOP = X.IDTIPOORCAOP
                                    AND S.CDSERVFILI = X.CDSERVFILI
                                    AND S.DTORCAMENTO = X.DTORCAMENTO
                                    AND S.NRSIMUORCAGOP BETWEEN C_BUDGET.ANTERIOR AND C_BUDGET.NRSIMUORCAGOP-1),
                X.QTORCAMENTO = (SELECT QTORCAMENTO
                                   FROM SIMUORCAGOP S
                                  WHERE S.CDFILIAL = X.CDFILIAL
                                    AND S.IDTIPOORCAOP = X.IDTIPOORCAOP
                                    AND S.CDSERVFILI = X.CDSERVFILI
                                    AND S.DTORCAMENTO = X.DTORCAMENTO
                                    AND S.NRSIMUORCAGOP BETWEEN C_BUDGET.ANTERIOR AND C_BUDGET.NRSIMUORCAGOP-1);
                                     
         UPDATE (SELECT M.VRORCAMENTO, M.NRITORCAMESGOP
                   FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                               AND O.CDFILIAL = I.CDFILIAL
                                                               AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                      'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                       INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                  WHERE O.CDFILIAL = C_UNIDADE.CDFILIAL
                    AND O.NRANOORCAGOP = V_NRANOORCAGOP
                    AND O.IDPERIODOORCA = V_IDPERIODOORCA
                    AND O.NRPERIODOORCA = V_NRPERIODOORCA) X
            SET X.VRORCAMENTO = (SELECT SUM(D.VRORCAMENTO)
                                   FROM ITDETALORCAMESGOP D
                                  WHERE D.NRITORCAMESGOP = X.NRITORCAMESGOP);
                                    
         UPDATE (SELECT D.VRORCAMENTO, D.VRPERCENTUAL, M.VRORCAMENTO TOTAL
                   FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                               AND O.CDFILIAL = I.CDFILIAL
                                                               AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                      'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                       INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                       INNER JOIN ITDETALORCAMESGOP D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                  WHERE O.CDFILIAL = C_UNIDADE.CDFILIAL
                    AND O.NRANOORCAGOP = V_NRANOORCAGOP
                    AND O.IDPERIODOORCA = V_IDPERIODOORCA
                    AND O.NRPERIODOORCA = V_NRPERIODOORCA) X
            SET X.VRPERCENTUAL = CASE WHEN NVL(X.TOTAL,0) = 0 THEN 0 ELSE (X.VRORCAMENTO*100)/X.TOTAL END;
   
         UPDATE (SELECT I.CDINVEST, I.CDIMPOSTO, M.VRORCAMENTO, M.VRTOTALMES, M.DTORCAMENTO, I.IDTIPOORCAOP, 
                        O.CDFILIAL, M.VRPERCENTUAL, I.CDSERVFILI
                   FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                               AND I.IDTIPOORCAOP IN ('INVESTIMENTO', 'DEPRECIACAO', 'TAXAADM',
                                                                                      'IMPOSTOSOBLUCRO', 'IMPOSTO', 'IMPOSTOARECUPERAR')
                                       INNER JOIN ITORCAMESGOP M ON I.NRITEMORCAOP = M.NRITEMORCAOP
                  WHERE O.CDFILIAL = C_UNIDADE.CDFILIAL
                    AND O.NRANOORCAGOP = V_NRANOORCAGOP
                    AND O.IDPERIODOORCA = V_IDPERIODOORCA
                    AND O.NRPERIODOORCA = V_NRPERIODOORCA) X
            SET X.VRORCAMENTO = (SELECT VRORCAMENTO
                                   FROM SIMUORCAGOP S
                                  WHERE S.CDFILIAL = X.CDFILIAL
                                    AND S.IDTIPOORCAOP = X.IDTIPOORCAOP
                                    AND ((S.CDINVEST = X.CDINVEST) OR (S.IDTIPOORCAOP IN ('DEPRECIACAO', 'TAXAADM'))
                                        OR (X.CDIMPOSTO = S.CODIGO))
                                    AND S.DTORCAMENTO = X.DTORCAMENTO
                                    AND S.NRSIMUORCAGOP BETWEEN C_BUDGET.ANTERIOR AND C_BUDGET.NRSIMUORCAGOP-1);
                                     
         UPDATE (SELECT M.VRORCAMENTO, M.VRPERCENTUAL,
                        (SELECT SUM(MM.VRORCAMENTO) TOTAL
                           FROM ITEMORCAGOP II INNER JOIN ITORCAMESGOP   MM ON II.NRITEMORCAOP = MM.NRITEMORCAOP
                          WHERE II.IDTIPOORCAOP = I.IDTIPOORCAOP
                            AND MM.DTORCAMENTO = M.DTORCAMENTO
                            AND II.CDFILIAL = I.CDFILIAL) TOTAL
                   FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                               AND O.CDFILIAL = I.CDFILIAL
                                       INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                  WHERE O.CDFILIAL = C_UNIDADE.CDFILIAL
                    AND O.NRANOORCAGOP = V_NRANOORCAGOP
                    AND O.IDPERIODOORCA = V_IDPERIODOORCA
                    AND O.NRPERIODOORCA = V_NRPERIODOORCA) X
            SET X.VRPERCENTUAL = CASE WHEN NVL(X.TOTAL, 0) = 0 THEN 0
                                      ELSE (X.VRORCAMENTO * 100) / X.TOTAL
                                 END;
      END LOOP;
      
      /* PERCENT */
      IF C_BUDGET.CODIGO = 'BUDGETCHANGED' THEN
         FOR C_UNIDADE IN (SELECT CDLISTA CODIGO
                             FROM LISTA
                            WHERE NRLISTA = C_BUDGET.NRLISTA) LOOP
            BEGIN
               SELECT TO_CHAR(P.DTINIPERIORCA,'YYYY'), P.IDPERIODOORCA, P.NRPERIODO
                 INTO V_NRANOORCAGOP, V_IDPERIODOORCA, V_NRPERIODOORCA
                 FROM PERIODOORCAGOP P
                WHERE P.CDFILIAL = C_UNIDADE.CODIGO
                  --AND C_BUDGET.DTORCAMENTO BETWEEN P.DTINIPERIORCA AND P.DTFIMPERIORCA
                GROUP BY TO_CHAR(P.DTINIPERIORCA,'YYYY'), P.IDPERIODOORCA, P.NRPERIODO;
            EXCEPTION
              WHEN NO_DATA_FOUND THEN
                V_NRANOORCAGOP := NULL; V_IDPERIODOORCA := NULL; V_NRPERIODOORCA := NULL;
            END;
            
            UPDATE (SELECT D.CDSERVFILI, D.VRPRECOBASE, D.NRDIASUTEIS, D.VRORCAMENTO, D.QTORCAMENTO, D.VRPERCENTUAL,
                           O.CDFILIAL, I.IDTIPOORCAOP, M.DTORCAMENTO, D.CDPRODUTO, I.CDDESPPREV, D.CDDESPADMFAT, D.CDORCAMENTOMO, 
                           CASE WHEN I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'MAOOBRA') 
                                THEN (SELECT COUNT(1)
                                        FROM ITEMORCAGOP T INNER JOIN ITORCAMESGOP TM ON T.NRITEMORCAOP = TM.NRITEMORCAOP
                                                           INNER JOIN ITDETALORCAMESGOP TD ON TD.NRITORCAMESGOP = TM.NRITORCAMESGOP
                                       WHERE T.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                         AND T.IDTIPOORCAOP = I.IDTIPOORCAOP
                                         AND NOT EXISTS (SELECT 1
                                                           FROM ITEMORCAGOP T INNER JOIN ITORCAMESGOP TM ON T.NRITEMORCAOP = TM.NRITEMORCAOP
                                                                              INNER JOIN ITDETALORCAMESGOP TD ON TD.NRITORCAMESGOP = TM.NRITORCAMESGOP
                                                          WHERE T.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                            AND T.IDTIPOORCAOP = I.IDTIPOORCAOP
                                                            AND NVL(TD.VRORCAMENTO,0) <> 0))
                                ELSE (SELECT CASE WHEN COUNT(1) = 0 THEN 1 ELSE COUNT(1) END
                                        FROM ITDETALORCAMESGOP T
                                       WHERE T.NRITORCAMESGOP = M.NRITORCAMESGOP)
                           END FATOR
                      FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                  AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                         'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                          INNER JOIN ITORCAMESGOP M ON I.NRITEMORCAOP = M.NRITEMORCAOP
                                          INNER JOIN ITDETALORCAMESGOP D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                     WHERE O.CDFILIAL = C_UNIDADE.CODIGO
                       AND O.NRANOORCAGOP = V_NRANOORCAGOP
                       AND O.IDPERIODOORCA = V_IDPERIODOORCA
                       AND O.NRPERIODOORCA = V_NRPERIODOORCA
                       AND I.IDTIPOORCAOP = C_BUDGET.IDTIPOORCAOP) X
               SET X.VRORCAMENTO = CASE WHEN X.FATOR > 0
                                        THEN C_BUDGET.VRORCAMENTO/X.FATOR
                                        ELSE X.VRORCAMENTO*C_BUDGET.VRAJUSTPERCENTUAL
                                   END,
                   X.QTORCAMENTO = CASE WHEN X.FATOR > 0
                                        THEN C_BUDGET.QTORCAMENTO/X.FATOR
                                        ELSE X.QTORCAMENTO*C_BUDGET.VRAJUSTPERCENTUAL
                                   END;
                                     
            UPDATE (SELECT M.VRORCAMENTO, M.NRITORCAMESGOP, I.NRITEMORCAOP
                      FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                                  AND O.CDFILIAL = I.CDFILIAL
                                                                  AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                         'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                          INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                     WHERE O.CDFILIAL = C_UNIDADE.CODIGO
                       AND O.NRANOORCAGOP = V_NRANOORCAGOP
                       AND O.IDPERIODOORCA = V_IDPERIODOORCA
                       AND O.NRPERIODOORCA = V_NRPERIODOORCA
                       AND I.IDTIPOORCAOP = C_BUDGET.IDTIPOORCAOP) X
               SET X.VRORCAMENTO = (SELECT SUM(D.VRORCAMENTO)
                                      FROM ITDETALORCAMESGOP D
                                     WHERE D.NRITORCAMESGOP = X.NRITORCAMESGOP);
                                    
            UPDATE (SELECT D.VRORCAMENTO, D.VRPERCENTUAL, M.VRORCAMENTO TOTAL, M.NRITORCAMESGOP
                      FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                                  AND O.CDFILIAL = I.CDFILIAL
                                                                  AND I.IDTIPOORCAOP IN ('FATURAMENTO', 'CONSUMO', 'DESPMATRIZ',
                                                                                         'DESPENTRADA', 'DESPDIVERSA', 'MAOOBRA')
                                          INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                          INNER JOIN ITDETALORCAMESGOP D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                     WHERE O.CDFILIAL = C_UNIDADE.CODIGO
                       AND O.NRANOORCAGOP = V_NRANOORCAGOP
                       AND O.IDPERIODOORCA = V_IDPERIODOORCA
                       AND O.NRPERIODOORCA = V_NRPERIODOORCA
                       AND I.IDTIPOORCAOP = C_BUDGET.IDTIPOORCAOP) X
               SET X.VRPERCENTUAL = CASE WHEN NVL(X.TOTAL,0) = 0 THEN 0 ELSE (X.VRORCAMENTO*100)/X.TOTAL END;
   
            UPDATE (SELECT I.CDINVEST, I.CDIMPOSTO, M.VRORCAMENTO, M.VRTOTALMES, M.DTORCAMENTO, I.IDTIPOORCAOP, 
                           O.CDFILIAL, M.VRPERCENTUAL, I.CDSERVFILI, I.NRITEMORCAOP
                      FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                  AND I.IDTIPOORCAOP IN ('INVESTIMENTO', 'DEPRECIACAO', 'TAXAADM',
                                                                                         'IMPOSTOSOBLUCRO', 'IMPOSTO', 'IMPOSTOARECUPERAR')
                                          INNER JOIN ITORCAMESGOP M ON I.NRITEMORCAOP = M.NRITEMORCAOP
                     WHERE O.CDFILIAL = C_UNIDADE.CODIGO
                       AND O.NRANOORCAGOP = V_NRANOORCAGOP
                       AND O.IDPERIODOORCA = V_IDPERIODOORCA
                       AND O.NRPERIODOORCA = V_NRPERIODOORCA
                       AND I.IDTIPOORCAOP = C_BUDGET.IDTIPOORCAOP) X
               SET X.VRORCAMENTO = CASE WHEN NVL(X.VRORCAMENTO,0) = 0
                                        THEN C_BUDGET.VRORCAMENTO
                                        ELSE X.VRORCAMENTO * C_BUDGET.VRAJUSTPERCENTUAL
                                   END;
                                     
            UPDATE (SELECT M.VRORCAMENTO, M.VRPERCENTUAL,
                           (SELECT SUM(MM.VRORCAMENTO) TOTAL
                              FROM ITEMORCAGOP II INNER JOIN ITORCAMESGOP   MM ON II.NRITEMORCAOP = MM.NRITEMORCAOP
                             WHERE II.IDTIPOORCAOP = I.IDTIPOORCAOP
                               AND MM.DTORCAMENTO = M.DTORCAMENTO
                               AND II.CDFILIAL = I.CDFILIAL) TOTAL, I.NRITEMORCAOP
                      FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                                  AND O.CDFILIAL = I.CDFILIAL
                                          INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                     WHERE O.CDFILIAL = C_UNIDADE.CODIGO
                       AND O.NRANOORCAGOP = V_NRANOORCAGOP
                       AND O.IDPERIODOORCA = V_IDPERIODOORCA
                       AND O.NRPERIODOORCA = V_NRPERIODOORCA
                       AND I.IDTIPOORCAOP = C_BUDGET.IDTIPOORCAOP) X
               SET X.VRPERCENTUAL = CASE WHEN NVL(X.TOTAL, 0) = 0 THEN 0
                                         ELSE (X.VRORCAMENTO * 100) / X.TOTAL
                                    END;
         END LOOP;
      END IF;
      
      COMMIT;
   END LOOP;
                           
   DELETE FROM SIMUORCAGOP;
   
   COMMIT;
END;
";
        try{
            $std = $this->connection->prepare($sqlScript);
            $std->execute();
        }catch(Exception $e) {
            throw $e;
        }
    }
 } 